@@ -1,8 +1,8 @@
 /*
- * Copyright (C) 1995-2006, Index Data ApS
+ * Copyright (C) 1995-2007, Index Data ApS
  * See the file LICENSE for details.
  *
- * $Id: mime.h,v 1.2 2006/04/20 20:50:51 adam Exp $
+ * $Id: mime.h,v 1.4 2007/01/03 08:42:15 adam Exp $
  */
 
 /** \file mime.h
@@ -14,7 +14,7 @@
 
 typedef struct yaz_mime_info *yaz_mime_types;
 
-yaz_mime_types yaz_mime_types_create();
+yaz_mime_types yaz_mime_types_create(void);
 void yaz_mime_types_add(yaz_mime_types t, const char *suffix,
                         const char *mime_type);
 const char *yaz_mime_lookup_suffix(yaz_mime_types t, const char *suffix);
