@@ -1,8 +1,8 @@
 /*
- * Copyright (C) 1995-2005, Index Data ApS
+ * Copyright (C) 1995-2007, Index Data ApS
  * See the file LICENSE for details.
  *
- * $Id: wrbuf.c,v 1.10 2006/04/19 10:05:03 adam Exp $
+ * $Id: wrbuf.c,v 1.16 2007/02/23 10:15:01 adam Exp $
  */
 
 /**
@@ -20,6 +20,7 @@
 #include <stdarg.h>
 
 #include <yaz/wrbuf.h>
+#include <yaz/snprintf.h>
 #include <yaz/yaz-iconv.h>
 
 WRBUF wrbuf_alloc(void)
@@ -41,6 +42,11 @@ void wrbuf_free(WRBUF b, int free_buf)
     xfree(b);
 }
 
+void wrbuf_destroy(WRBUF b)
+{
+    wrbuf_free(b, 1);
+}
+
 void wrbuf_rewind(WRBUF b)
 {
     b->pos = 0;
@@ -81,6 +87,21 @@ int wrbuf_puts(WRBUF b, const char *buf)
     return 0;
 }
 
+int wrbuf_puts_replace_char(WRBUF b, const char *buf, 
+                            const char from, const char to)
+{
+    while(*buf){
+        if (*buf == from)
+            wrbuf_putc(b, to);
+        else
+            wrbuf_putc(b, *buf);
+        buf++;
+    }
+    wrbuf_putc(b, 0);
+    (b->pos)--;                          /* don't include '\0' in count */
+    return 0;
+}
+
 void wrbuf_chop_right(WRBUF b)
 {
     while (b->pos && b->buf[b->pos-1] == ' ')
@@ -139,16 +160,7 @@ void wrbuf_printf(WRBUF b, const char *fmt, ...)
     char buf[4096];
 
     va_start(ap, fmt);
-#ifdef WIN32
-    _vsnprintf(buf, sizeof(buf)-1, fmt, ap);
-#else
-/* !WIN32 */
-#if HAVE_VSNPRINTF
-    vsnprintf(buf, sizeof(buf)-1, fmt, ap);
-#else
-    vsprintf(buf, fmt, ap);
-#endif
-#endif
+    yaz_vsnprintf(buf, sizeof(buf)-1, fmt, ap);
     wrbuf_puts (b, buf);
 
     va_end(ap);
@@ -200,11 +212,25 @@ int wrbuf_iconv_puts(WRBUF b, yaz_iconv_t cd, const char *strz)
     return wrbuf_iconv_write(b, cd, strz, strlen(strz));
 }
 
+int wrbuf_iconv_putchar(WRBUF b, yaz_iconv_t cd, int ch)
+{
+    char buf[1];
+    buf[0] = ch;
+    return wrbuf_iconv_write(b, cd, buf, 1);
+}
+
 int wrbuf_iconv_write_cdata(WRBUF b, yaz_iconv_t cd, const char *buf, int size)
 {
     return wrbuf_iconv_write_x(b, cd, buf, size, 1);
 }
 
+const char *wrbuf_cstr(WRBUF b)
+{
+    wrbuf_write(b, "", 1);  /* '\0'-terminate as well */
+    (b->pos)--;             /* don't include '\0' in count */
+    return b->buf;
+}
+
 /*
  * Local variables:
  * c-basic-offset: 4
