@@ -1,10 +1,12 @@
 #!/bin/sh
-# $Id: tstmarcxml.sh,v 1.2 2006/04/20 19:47:02 adam Exp $
+# $Id: tstmarcxml.sh,v 1.5 2007/02/17 10:53:05 adam Exp $
 # Tests reading of MARCXML and checks that we get identical ISO2709 output.
+#
+# Reads marc?.xml files , Generates marc?.xml.marc files
 srcdir=${srcdir:-.}
 ecode=0
 # Skip this test if Libxml2 support is not enabled
-../util/yaz-marcdump -x >/dev/null 2>&1
+../util/yaz-marcdump -i marcxml >/dev/null 2>&1
 if test $? = "3"; then
     exit 0
 fi
@@ -12,7 +14,7 @@ for f in ${srcdir}/marc?.xml; do
     NEW=`basename ${f}`.new.marc
     OLD=${f}.marc
     DIFF=`basename ${f}`.diff
-    ../util/yaz-marcdump -f utf-8 -t utf-8 -x -I $f > $NEW
+    ../util/yaz-marcdump -f utf-8 -t utf-8 -i marcxml -o marc $f > $NEW
     if test $? != "0"; then
 	echo "Failed decode of $f"
 	ecode=1
@@ -31,3 +33,8 @@ for f in ${srcdir}/marc?.xml; do
 done
 exit $ecode
 
+# Local Variables:
+# mode:shell-script
+# sh-indentation: 2
+# sh-basic-offset: 4
+# End:
