@@ -1,19 +1,19 @@
 /*
- * Copyright (C) 1995-2005, Index Data ApS
+ * Copyright (C) 1995-2007, Index Data ApS
  * See the file LICENSE for details.
  *
- * $Id: tstsoap1.c,v 1.6 2006/01/30 16:58:06 adam Exp $
+ * $Id: tstsoap1.c,v 1.9 2007/01/03 08:42:16 adam Exp $
  */
 
 #include <stdlib.h>
-#if HAVE_XML2
+#if YAZ_HAVE_XML2
 #include <libxml/parser.h>
 #endif
 #include <yaz/test.h>
 
-void tst()
+void tst(void)
 {
-#if HAVE_XML2
+#if YAZ_HAVE_XML2
     xmlChar *buf_out;
     int len_out;
     xmlDocPtr doc;
@@ -38,14 +38,14 @@ void tst()
 #endif
 
 
-/* HAVE_XML2 */
+/* YAZ_HAVE_XML2 */
 #endif
 }
 
 int main(int argc, char **argv)
 {
     YAZ_CHECK_INIT(argc, argv);
-#if HAVE_XML2
+#if YAZ_HAVE_XML2
     LIBXML_TEST_VERSION;
 #endif
     tst();
